class ZoneinfoError(Exception):

    pass


class InvalidZoneinfoFile(ZoneinfoError):

    pass


class InvalidTimezone(ZoneinfoError):
    def __init__(self, name):
        super(InvalidTimezone, self).__init__('Invalid timezone "{}"'.format(name))


class InvalidPosixSpec(ZoneinfoError):
    def __init__(self, spec):
        super(InvalidPosixSpec, self).__init__("Invalid POSIX spec: {}".format(spec))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .custom import translations as custom_translations


"""
fa locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one"
    if ((n == n and ((n == 0))) or (n == n and ((n == 1))))
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "یکشنبه",
                1: "دوشنبه",
                2: "سه\u200cشنبه",
                3: "چهارشنبه",
                4: "پنجشنبه",
                5: "جمعه",
                6: "شنبه",
            },
            "narrow": {0: "ی", 1: "د", 2: "س", 3: "چ", 4: "پ", 5: "ج", 6: "ش"},
            "short": {0: "۱ش", 1: "۲ش", 2: "۳ش", 3: "۴ش", 4: "۵ش", 5: "ج", 6: "ش"},
            "wide": {
                0: "یکشنبه",
                1: "دوشنبه",
                2: "سه\u200cشنبه",
                3: "چهارشنبه",
                4: "پنجشنبه",
                5: "جمعه",
                6: "شنبه",
            },
        },
        "months": {
            "abbreviated": {
                1: "ژانویهٔ",
                2: "فوریهٔ",
                3: "مارس",
                4: "آوریل",
                5: "مهٔ",
                6: "ژوئن",
                7: "ژوئیهٔ",
                8: "اوت",
                9: "سپتامبر",
                10: "اکتبر",
                11: "نوامبر",
                12: "دسامبر",
            },
            "narrow": {
                1: "ژ",
                2: "ف",
                3: "م",
                4: "آ",
                5: "م",
                6: "ژ",
                7: "ژ",
                8: "ا",
                9: "س",
                10: "ا",
                11: "ن",
                12: "د",
            },
            "wide": {
                1: "ژانویهٔ",
                2: "فوریهٔ",
                3: "مارس",
                4: "آوریل",
                5: "مهٔ",
                6: "ژوئن",
                7: "ژوئیهٔ",
                8: "اوت",
                9: "سپتامبر",
                10: "اکتبر",
                11: "نوامبر",
                12: "دسامبر",
            },
        },
        "units": {
            "year": {"one": "{0} سال", "other": "{0} سال"},
            "month": {"one": "{0} ماه", "other": "{0} ماه"},
            "week": {"one": "{0} هفته", "other": "{0} هفته"},
            "day": {"one": "{0} روز", "other": "{0} روز"},
            "hour": {"one": "{0} ساعت", "other": "{0} ساعت"},
            "minute": {"one": "{0} دقیقه", "other": "{0} دقیقه"},
            "second": {"one": "{0} ثانیه", "other": "{0} ثانیه"},
            "microsecond": {"one": "{0} میکروثانیه", "other": "{0} میکروثانیه"},
        },
        "relative": {
            "year": {
                "future": {"other": "{0} سال بعد", "one": "{0} سال بعد"},
                "past": {"other": "{0} سال پیش", "one": "{0} سال پیش"},
            },
            "month": {
                "future": {"other": "{0} ماه بعد", "one": "{0} ماه بعد"},
                "past": {"other": "{0} ماه پیش", "one": "{0} ماه پیش"},
            },
            "week": {
                "future": {"other": "{0} هفته بعد", "one": "{0} هفته بعد"},
                "past": {"other": "{0} هفته پیش", "one": "{0} هفته پیش"},
            },
            "day": {
                "future": {"other": "{0} روز بعد", "one": "{0} روز بعد"},
                "past": {"other": "{0} روز پیش", "one": "{0} روز پیش"},
            },
            "hour": {
                "future": {"other": "{0} ساعت بعد", "one": "{0} ساعت بعد"},
                "past": {"other": "{0} ساعت پیش", "one": "{0} ساعت پیش"},
            },
            "minute": {
                "future": {"other": "{0} دقیقه بعد", "one": "{0} دقیقه بعد"},
                "past": {"other": "{0} دقیقه پیش", "one": "{0} دقیقه پیش"},
            },
            "second": {
                "future": {"other": "{0} ثانیه بعد", "one": "{0} ثانیه بعد"},
                "past": {"other": "{0} ثانیه پیش", "one": "{0} ثانیه پیش"},
            },
        },
        "day_periods": {
            "midnight": "نیمه\u200cشب",
            "am": "قبل\u200cازظهر",
            "noon": "ظهر",
            "pm": "بعدازظهر",
            "morning1": "صبح",
            "afternoon1": "عصر",
            "evening1": "عصر",
            "night1": "شب",
        },
    },
    "custom": custom_translations,
}



from typing import Tuple
from typing import Union

import pytzdata

from .local_timezone import get_local_timezone
from .local_timezone import set_local_timezone
from .local_timezone import test_local_timezone
from .timezone import UTC
from .timezone import FixedTimezone as _FixedTimezone
from .timezone import Timezone as _Timezone


PRE_TRANSITION = "pre"
POST_TRANSITION = "post"
TRANSITION_ERROR = "error"

timezones = pytzdata.timezones  # type: Tuple[str, ...]


_tz_cache = {}


def timezone(name, extended=True):  # type: (Union[str, int], bool) -> _Timezone
    """
    Return a Timezone instance given its name.
    """
    if isinstance(name, int):
        return fixed_timezone(name)

    if name.lower() == "utc":
        return UTC

    if name in _tz_cache:
        return _tz_cache[name]

    tz = _Timezone(name, extended=extended)
    _tz_cache[name] = tz

    return tz


def fixed_timezone(offset):  # type: (int) -> _FixedTimezone
    """
    Return a Timezone instance given its offset in seconds.
    """
    if offset in _tz_cache:
        return _tz_cache[offset]  # type: ignore

    tz = _FixedTimezone(offset)
    _tz_cache[offset] = tz

    return tz


def local_timezone():  # type: () -> _Timezone
    """
    Return the local timezone.
    """
    return get_local_timezone()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
de custom locale file.
"""

translations = {
    # Relative time
    "after": "{0} später",
    "before": "{0} zuvor",
    "units_relative": {
        "year": {
            "future": {"one": "{0} Jahr", "other": "{0} Jahren"},
            "past": {"one": "{0} Jahr", "other": "{0} Jahren"},
        },
        "month": {
            "future": {"one": "{0} Monat", "other": "{0} Monaten"},
            "past": {"one": "{0} Monat", "other": "{0} Monaten"},
        },
        "week": {
            "future": {"one": "{0} Woche", "other": "{0} Wochen"},
            "past": {"one": "{0} Woche", "other": "{0} Wochen"},
        },
        "day": {
            "future": {"one": "{0} Tag", "other": "{0} Tagen"},
            "past": {"one": "{0} Tag", "other": "{0} Tagen"},
        },
    },
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "LLLL": "dddd, D. MMMM YYYY HH:mm",
        "LLL": "D. MMMM YYYY HH:mm",
        "LL": "D. MMMM YYYY",
        "L": "DD.MM.YYYY",
    },
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .custom import translations as custom_translations


"""
ru locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "few"
    if (
        (
            (0 == 0 and ((0 == 0)))
            and ((n % 10) == (n % 10) and (((n % 10) >= 2 and (n % 10) <= 4)))
        )
        and (not ((n % 100) == (n % 100) and (((n % 100) >= 12 and (n % 100) <= 14))))
    )
    else "many"
    if (
        (
            ((0 == 0 and ((0 == 0))) and ((n % 10) == (n % 10) and (((n % 10) == 0))))
            or (
                (0 == 0 and ((0 == 0)))
                and ((n % 10) == (n % 10) and (((n % 10) >= 5 and (n % 10) <= 9)))
            )
        )
        or (
            (0 == 0 and ((0 == 0)))
            and ((n % 100) == (n % 100) and (((n % 100) >= 11 and (n % 100) <= 14)))
        )
    )
    else "one"
    if (
        ((0 == 0 and ((0 == 0))) and ((n % 10) == (n % 10) and (((n % 10) == 1))))
        and (not ((n % 100) == (n % 100) and (((n % 100) == 11))))
    )
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "вс",
                1: "пн",
                2: "вт",
                3: "ср",
                4: "чт",
                5: "пт",
                6: "сб",
            },
            "narrow": {0: "вс", 1: "пн", 2: "вт", 3: "ср", 4: "чт", 5: "пт", 6: "сб"},
            "short": {0: "вс", 1: "пн", 2: "вт", 3: "ср", 4: "чт", 5: "пт", 6: "сб"},
            "wide": {
                0: "воскресенье",
                1: "понедельник",
                2: "вторник",
                3: "среда",
                4: "четверг",
                5: "пятница",
                6: "суббота",
            },
        },
        "months": {
            "abbreviated": {
                1: "янв.",
                2: "февр.",
                3: "мар.",
                4: "апр.",
                5: "мая",
                6: "июн.",
                7: "июл.",
                8: "авг.",
                9: "сент.",
                10: "окт.",
                11: "нояб.",
                12: "дек.",
            },
            "narrow": {
                1: "Я",
                2: "Ф",
                3: "М",
                4: "А",
                5: "М",
                6: "И",
                7: "И",
                8: "А",
                9: "С",
                10: "О",
                11: "Н",
                12: "Д",
            },
            "wide": {
                1: "января",
                2: "февраля",
                3: "марта",
                4: "апреля",
                5: "мая",
                6: "июня",
                7: "июля",
                8: "августа",
                9: "сентября",
                10: "октября",
                11: "ноября",
                12: "декабря",
            },
        },
        "units": {
            "year": {
                "one": "{0} год",
                "few": "{0} года",
                "many": "{0} лет",
                "other": "{0} года",
            },
            "month": {
                "one": "{0} месяц",
                "few": "{0} месяца",
                "many": "{0} месяцев",
                "other": "{0} месяца",
            },
            "week": {
                "one": "{0} неделя",
                "few": "{0} недели",
                "many": "{0} недель",
                "other": "{0} недели",
            },
            "day": {
                "one": "{0} день",
                "few": "{0} дня",
                "many": "{0} дней",
                "other": "{0} дня",
            },
            "hour": {
                "one": "{0} час",
                "few": "{0} часа",
                "many": "{0} часов",
                "other": "{0} часа",
            },
            "minute": {
                "one": "{0} минута",
                "few": "{0} минуты",
                "many": "{0} минут",
                "other": "{0} минуты",
            },
            "second": {
                "one": "{0} секунда",
                "few": "{0} секунды",
                "many": "{0} секунд",
                "other": "{0} секунды",
            },
            "microsecond": {
                "one": "{0} микросекунда",
                "few": "{0} микросекунды",
                "many": "{0} микросекунд",
                "other": "{0} микросекунды",
            },
        },
        "relative": {
            "year": {
                "future": {
                    "other": "через {0} года",
                    "one": "через {0} год",
                    "few": "через {0} года",
                    "many": "через {0} лет",
                },
                "past": {
                    "other": "{0} года назад",
                    "one": "{0} год назад",
                    "few": "{0} года назад",
                    "many": "{0} лет назад",
                },
            },
            "month": {
                "future": {
                    "other": "через {0} месяца",
                    "one": "через {0} месяц",
                    "few": "через {0} месяца",
                    "many": "через {0} месяцев",
                },
                "past": {
                    "other": "{0} месяца назад",
                    "one": "{0} месяц назад",
                    "few": "{0} месяца назад",
                    "many": "{0} месяцев назад",
                },
            },
            "week": {
                "future": {
                    "other": "через {0} недели",
                    "one": "через {0} неделю",
                    "few": "через {0} недели",
                    "many": "через {0} недель",
                },
                "past": {
                    "other": "{0} недели назад",
                    "one": "{0} неделю назад",
                    "few": "{0} недели назад",
                    "many": "{0} недель назад",
                },
            },
            "day": {
                "future": {
                    "other": "через {0} дня",
                    "one": "через {0} день",
                    "few": "через {0} дня",
                    "many": "через {0} дней",
                },
                "past": {
                    "other": "{0} дня назад",
                    "one": "{0} день назад",
                    "few": "{0} дня назад",
                    "many": "{0} дней назад",
                },
            },
            "hour": {
                "future": {
                    "other": "через {0} часа",
                    "one": "через {0} час",
                    "few": "через {0} часа",
                    "many": "через {0} часов",
                },
                "past": {
                    "other": "{0} часа назад",
                    "one": "{0} час назад",
                    "few": "{0} часа назад",
                    "many": "{0} часов назад",
                },
            },
            "minute": {
                "future": {
                    "other": "через {0} минуты",
                    "one": "через {0} минуту",
                    "few": "через {0} минуты",
                    "many": "через {0} минут",
                },
                "past": {
                    "other": "{0} минуты назад",
                    "one": "{0} минуту назад",
                    "few": "{0} минуты назад",
                    "many": "{0} минут назад",
                },
            },
            "second": {
                "future": {
                    "other": "через {0} секунды",
                    "one": "через {0} секунду",
                    "few": "через {0} секунды",
                    "many": "через {0} секунд",
                },
                "past": {
                    "other": "{0} секунды назад",
                    "one": "{0} секунду назад",
                    "few": "{0} секунды назад",
                    "many": "{0} секунд назад",
                },
            },
        },
        "day_periods": {
            "midnight": "полночь",
            "am": "AM",
            "noon": "полдень",
            "pm": "PM",
            "morning1": "утра",
            "afternoon1": "дня",
            "evening1": "вечера",
            "night1": "ночи",
        },
    },
    "custom": custom_translations,
}



from .reader import Reader
from .timezone import Timezone


def read(name, extend=True):  # type: (str, bool) -> Timezone
    """
    Read the zoneinfo structure for a given timezone name.
    """
    return Reader(extend=extend).read_for(name)


def read_file(path, extend=True):  # type: (str, bool) -> Timezone
    """
    Read the zoneinfo structure for a given path.
    """
    return Reader(extend=extend).read(path)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .custom import translations as custom_translations


"""
nl locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one"
    if ((n == n and ((n == 1))) and (0 == 0 and ((0 == 0))))
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "zo",
                1: "ma",
                2: "di",
                3: "wo",
                4: "do",
                5: "vr",
                6: "za",
            },
            "narrow": {0: "Z", 1: "M", 2: "D", 3: "W", 4: "D", 5: "V", 6: "Z"},
            "short": {0: "zo", 1: "ma", 2: "di", 3: "wo", 4: "do", 5: "vr", 6: "za"},
            "wide": {
                0: "zondag",
                1: "maandag",
                2: "dinsdag",
                3: "woensdag",
                4: "donderdag",
                5: "vrijdag",
                6: "zaterdag",
            },
        },
        "months": {
            "abbreviated": {
                1: "jan.",
                2: "feb.",
                3: "mrt.",
                4: "apr.",
                5: "mei",
                6: "jun.",
                7: "jul.",
                8: "aug.",
                9: "sep.",
                10: "okt.",
                11: "nov.",
                12: "dec.",
            },
            "narrow": {
                1: "J",
                2: "F",
                3: "M",
                4: "A",
                5: "M",
                6: "J",
                7: "J",
                8: "A",
                9: "S",
                10: "O",
                11: "N",
                12: "D",
            },
            "wide": {
                1: "januari",
                2: "februari",
                3: "maart",
                4: "april",
                5: "mei",
                6: "juni",
                7: "juli",
                8: "augustus",
                9: "september",
                10: "oktober",
                11: "november",
                12: "december",
            },
        },
        "units": {
            "year": {"one": "{0} jaar", "other": "{0} jaar"},
            "month": {"one": "{0} maand", "other": "{0} maanden"},
            "week": {"one": "{0} week", "other": "{0} weken"},
            "day": {"one": "{0} dag", "other": "{0} dagen"},
            "hour": {"one": "{0} uur", "other": "{0} uur"},
            "minute": {"one": "{0} minuut", "other": "{0} minuten"},
            "second": {"one": "{0} seconde", "other": "{0} seconden"},
            "microsecond": {"one": "{0} microseconde", "other": "{0} microseconden"},
        },
        "relative": {
            "year": {
                "future": {"other": "over {0} jaar", "one": "over {0} jaar"},
                "past": {"other": "{0} jaar geleden", "one": "{0} jaar geleden"},
            },
            "month": {
                "future": {"other": "over {0} maanden", "one": "over {0} maand"},
                "past": {"other": "{0} maanden geleden", "one": "{0} maand geleden"},
            },
            "week": {
                "future": {"other": "over {0} weken", "one": "over {0} week"},
                "past": {"other": "{0} weken geleden", "one": "{0} week geleden"},
            },
            "day": {
                "future": {"other": "over {0} dagen", "one": "over {0} dag"},
                "past": {"other": "{0} dagen geleden", "one": "{0} dag geleden"},
            },
            "hour": {
                "future": {"other": "over {0} uur", "one": "over {0} uur"},
                "past": {"other": "{0} uur geleden", "one": "{0} uur geleden"},
            },
            "minute": {
                "future": {"other": "over {0} minuten", "one": "over {0} minuut"},
                "past": {"other": "{0} minuten geleden", "one": "{0} minuut geleden"},
            },
            "second": {
                "future": {"other": "over {0} seconden", "one": "over {0} seconde"},
                "past": {"other": "{0} seconden geleden", "one": "{0} seconde geleden"},
            },
        },
        "day_periods": {
            "midnight": "middernacht",
            "am": "a.m.",
            "pm": "p.m.",
            "morning1": "‘s ochtends",
            "afternoon1": "‘s middags",
            "evening1": "‘s avonds",
            "night1": "‘s nachts",
        },
    },
    "custom": custom_translations,
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .custom import translations as custom_translations


"""
it locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one"
    if ((n == n and ((n == 1))) and (0 == 0 and ((0 == 0))))
    else "other",
    "ordinal": lambda n: "many"
    if (n == n and ((n == 11) or (n == 8) or (n == 80) or (n == 800)))
    else "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "dom",
                1: "lun",
                2: "mar",
                3: "mer",
                4: "gio",
                5: "ven",
                6: "sab",
            },
            "narrow": {0: "D", 1: "L", 2: "M", 3: "M", 4: "G", 5: "V", 6: "S"},
            "short": {
                0: "dom",
                1: "lun",
                2: "mar",
                3: "mer",
                4: "gio",
                5: "ven",
                6: "sab",
            },
            "wide": {
                0: "domenica",
                1: "lunedì",
                2: "martedì",
                3: "mercoledì",
                4: "giovedì",
                5: "venerdì",
                6: "sabato",
            },
        },
        "months": {
            "abbreviated": {
                1: "gen",
                2: "feb",
                3: "mar",
                4: "apr",
                5: "mag",
                6: "giu",
                7: "lug",
                8: "ago",
                9: "set",
                10: "ott",
                11: "nov",
                12: "dic",
            },
            "narrow": {
                1: "G",
                2: "F",
                3: "M",
                4: "A",
                5: "M",
                6: "G",
                7: "L",
                8: "A",
                9: "S",
                10: "O",
                11: "N",
                12: "D",
            },
            "wide": {
                1: "gennaio",
                2: "febbraio",
                3: "marzo",
                4: "aprile",
                5: "maggio",
                6: "giugno",
                7: "luglio",
                8: "agosto",
                9: "settembre",
                10: "ottobre",
                11: "novembre",
                12: "dicembre",
            },
        },
        "units": {
            "year": {"one": "{0} anno", "other": "{0} anni"},
            "month": {"one": "{0} mese", "other": "{0} mesi"},
            "week": {"one": "{0} settimana", "other": "{0} settimane"},
            "day": {"one": "{0} giorno", "other": "{0} giorni"},
            "hour": {"one": "{0} ora", "other": "{0} ore"},
            "minute": {"one": "{0} minuto", "other": "{0} minuti"},
            "second": {"one": "{0} secondo", "other": "{0} secondi"},
            "microsecond": {"one": "{0} microsecondo", "other": "{0} microsecondi"},
        },
        "relative": {
            "year": {
                "future": {"other": "tra {0} anni", "one": "tra {0} anno"},
                "past": {"other": "{0} anni fa", "one": "{0} anno fa"},
            },
            "month": {
                "future": {"other": "tra {0} mesi", "one": "tra {0} mese"},
                "past": {"other": "{0} mesi fa", "one": "{0} mese fa"},
            },
            "week": {
                "future": {"other": "tra {0} settimane", "one": "tra {0} settimana"},
                "past": {"other": "{0} settimane fa", "one": "{0} settimana fa"},
            },
            "day": {
                "future": {"other": "tra {0} giorni", "one": "tra {0} giorno"},
                "past": {"other": "{0} giorni fa", "one": "{0} giorno fa"},
            },
            "hour": {
                "future": {"other": "tra {0} ore", "one": "tra {0} ora"},
                "past": {"other": "{0} ore fa", "one": "{0} ora fa"},
            },
            "minute": {
                "future": {"other": "tra {0} minuti", "one": "tra {0} minuto"},
                "past": {"other": "{0} minuti fa", "one": "{0} minuto fa"},
            },
            "second": {
                "future": {"other": "tra {0} secondi", "one": "tra {0} secondo"},
                "past": {"other": "{0} secondi fa", "one": "{0} secondo fa"},
            },
        },
        "day_periods": {
            "midnight": "mezzanotte",
            "am": "AM",
            "noon": "mezzogiorno",
            "pm": "PM",
            "morning1": "di mattina",
            "afternoon1": "del pomeriggio",
            "evening1": "di sera",
            "night1": "di notte",
        },
    },
    "custom": custom_translations,
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .custom import translations as custom_translations


"""
nn locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one" if (n == n and ((n == 1))) else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "søn.",
                1: "mån.",
                2: "tys.",
                3: "ons.",
                4: "tor.",
                5: "fre.",
                6: "lau.",
            },
            "narrow": {0: "S", 1: "M", 2: "T", 3: "O", 4: "T", 5: "F", 6: "L"},
            "short": {
                0: "sø.",
                1: "må.",
                2: "ty.",
                3: "on.",
                4: "to.",
                5: "fr.",
                6: "la.",
            },
            "wide": {
                0: "søndag",
                1: "måndag",
                2: "tysdag",
                3: "onsdag",
                4: "torsdag",
                5: "fredag",
                6: "laurdag",
            },
        },
        "months": {
            "abbreviated": {
                1: "jan.",
                2: "feb.",
                3: "mars",
                4: "apr.",
                5: "mai",
                6: "juni",
                7: "juli",
                8: "aug.",
                9: "sep.",
                10: "okt.",
                11: "nov.",
                12: "des.",
            },
            "narrow": {
                1: "J",
                2: "F",
                3: "M",
                4: "A",
                5: "M",
                6: "J",
                7: "J",
                8: "A",
                9: "S",
                10: "O",
                11: "N",
                12: "D",
            },
            "wide": {
                1: "januar",
                2: "februar",
                3: "mars",
                4: "april",
                5: "mai",
                6: "juni",
                7: "juli",
                8: "august",
                9: "september",
                10: "oktober",
                11: "november",
                12: "desember",
            },
        },
        "units": {
            "year": {"one": "{0} år", "other": "{0} år"},
            "month": {"one": "{0} månad", "other": "{0} månadar"},
            "week": {"one": "{0} veke", "other": "{0} veker"},
            "day": {"one": "{0} dag", "other": "{0} dagar"},
            "hour": {"one": "{0} time", "other": "{0} timar"},
            "minute": {"one": "{0} minutt", "other": "{0} minutt"},
            "second": {"one": "{0} sekund", "other": "{0} sekund"},
            "microsecond": {"one": "{0} mikrosekund", "other": "{0} mikrosekund"},
        },
        "relative": {
            "year": {
                "future": {"other": "om {0} år", "one": "om {0} år"},
                "past": {"other": "for {0} år sidan", "one": "for {0} år sidan"},
            },
            "month": {
                "future": {"other": "om {0} månadar", "one": "om {0} månad"},
                "past": {
                    "other": "for {0} månadar sidan",
                    "one": "for {0} månad sidan",
                },
            },
            "week": {
                "future": {"other": "om {0} veker", "one": "om {0} veke"},
                "past": {"other": "for {0} veker sidan", "one": "for {0} veke sidan"},
            },
            "day": {
                "future": {"other": "om {0} dagar", "one": "om {0} dag"},
                "past": {"other": "for {0} dagar sidan", "one": "for {0} dag sidan"},
            },
            "hour": {
                "future": {"other": "om {0} timar", "one": "om {0} time"},
                "past": {"other": "for {0} timar sidan", "one": "for {0} time sidan"},
            },
            "minute": {
                "future": {"other": "om {0} minutt", "one": "om {0} minutt"},
                "past": {
                    "other": "for {0} minutt sidan",
                    "one": "for {0} minutt sidan",
                },
            },
            "second": {
                "future": {"other": "om {0} sekund", "one": "om {0} sekund"},
                "past": {
                    "other": "for {0} sekund sidan",
                    "one": "for {0} sekund sidan",
                },
            },
        },
        "day_periods": {"am": "formiddag", "pm": "ettermiddag"},
    },
    "custom": custom_translations,
}



from __future__ import absolute_import

import os
import struct

from contextlib import contextmanager
from datetime import date
from datetime import datetime
from datetime import timedelta
from math import copysign
from typing import TYPE_CHECKING
from typing import Iterator
from typing import Optional
from typing import TypeVar
from typing import overload

import pendulum

from .constants import DAYS_PER_MONTHS
from .formatting.difference_formatter import DifferenceFormatter
from .locales.locale import Locale


if TYPE_CHECKING:
    # Prevent import cycles
    from .period import Period

with_extensions = os.getenv("PENDULUM_EXTENSIONS", "1") == "1"

_DT = TypeVar("_DT", bound=datetime)
_D = TypeVar("_D", bound=date)

try:
    if not with_extensions or struct.calcsize("P") == 4:
        raise ImportError()

    from ._extensions._helpers import local_time
    from ._extensions._helpers import precise_diff
    from ._extensions._helpers import is_leap
    from ._extensions._helpers import is_long_year
    from ._extensions._helpers import week_day
    from ._extensions._helpers import days_in_year
    from ._extensions._helpers import timestamp
except ImportError:
    from ._extensions.helpers import local_time  # noqa
    from ._extensions.helpers import precise_diff  # noqa
    from ._extensions.helpers import is_leap  # noqa
    from ._extensions.helpers import is_long_year  # noqa
    from ._extensions.helpers import week_day  # noqa
    from ._extensions.helpers import days_in_year  # noqa
    from ._extensions.helpers import timestamp  # noqa


difference_formatter = DifferenceFormatter()


@overload
def add_duration(
    dt,  # type: _DT
    years=0,  # type: int
    months=0,  # type: int
    weeks=0,  # type: int
    days=0,  # type: int
    hours=0,  # type: int
    minutes=0,  # type: int
    seconds=0,  # type: int
    microseconds=0,  # type: int
):  # type: (...) -> _DT
    pass


@overload
def add_duration(
    dt,  # type: _D
    years=0,  # type: int
    months=0,  # type: int
    weeks=0,  # type: int
    days=0,  # type: int
):  # type: (...) -> _D
    pass


def add_duration(
    dt,
    years=0,
    months=0,
    weeks=0,
    days=0,
    hours=0,
    minutes=0,
    seconds=0,
    microseconds=0,
):
    """
    Adds a duration to a date/datetime instance.
    """
    days += weeks * 7

    if (
        isinstance(dt, date)
        and not isinstance(dt, datetime)
        and any([hours, minutes, seconds, microseconds])
    ):
        raise RuntimeError("Time elements cannot be added to a date instance.")

    # Normalizing
    if abs(microseconds) > 999999:
        s = _sign(microseconds)
        div, mod = divmod(microseconds * s, 1000000)
        microseconds = mod * s
        seconds += div * s

    if abs(seconds) > 59:
        s = _sign(seconds)
        div, mod = divmod(seconds * s, 60)
        seconds = mod * s
        minutes += div * s

    if abs(minutes) > 59:
        s = _sign(minutes)
        div, mod = divmod(minutes * s, 60)
        minutes = mod * s
        hours += div * s

    if abs(hours) > 23:
        s = _sign(hours)
        div, mod = divmod(hours * s, 24)
        hours = mod * s
        days += div * s

    if abs(months) > 11:
        s = _sign(months)
        div, mod = divmod(months * s, 12)
        months = mod * s
        years += div * s

    year = dt.year + years
    month = dt.month

    if months:
        month += months
        if month > 12:
            year += 1
            month -= 12
        elif month < 1:
            year -= 1
            month += 12

    day = min(DAYS_PER_MONTHS[int(is_leap(year))][month], dt.day)

    dt = dt.replace(year=year, month=month, day=day)

    return dt + timedelta(
        days=days,
        hours=hours,
        minutes=minutes,
        seconds=seconds,
        microseconds=microseconds,
    )


def format_diff(
    diff, is_now=True, absolute=False, locale=None
):  # type: (Period, bool, bool, Optional[str]) -> str
    if locale is None:
        locale = get_locale()

    return difference_formatter.format(diff, is_now, absolute, locale)


def _sign(x):
    return int(copysign(1, x))


# Global helpers


@contextmanager
def test(mock):  # type: (pendulum.DateTime) -> Iterator[None]
    set_test_now(mock)
    try:
        yield
    finally:
        set_test_now()


def set_test_now(test_now=None):  # type: (Optional[pendulum.DateTime]) -> None
    pendulum._TEST_NOW = test_now


def get_test_now():  # type: () -> Optional[pendulum.DateTime]
    return pendulum._TEST_NOW


def has_test_now():  # type: () -> bool
    return pendulum._TEST_NOW is not None


def locale(name):  # type: (str) -> Locale
    return Locale.load(name)


def set_locale(name):  # type: (str) -> None
    locale(name)

    pendulum._LOCALE = name


def get_locale():  # type: () -> str
    return pendulum._LOCALE


def week_starts_at(wday):  # type: (int) -> None
    if wday < pendulum.SUNDAY or wday > pendulum.SATURDAY:
        raise ValueError("Invalid week day as start of week.")

    pendulum._WEEK_STARTS_AT = wday


def week_ends_at(wday):  # type: (int) -> None
    if wday < pendulum.SUNDAY or wday > pendulum.SATURDAY:
        raise ValueError("Invalid week day as start of week.")

    pendulum._WEEK_ENDS_AT = wday
